########################################################################
# Introscope EPAgent Plugin Script                               
#                                                                      
# Wily Technology (tm) Introscope(R) Version 4 Release 4.1
# Copyright (c) 1998 - 2003 Wily Technology, Inc. All Rights Reserved.
# Introscope(R) is a registered trademark of Wily Technology, Inc.
#
########################################################################
# vmStats.pl for HPUX
# -------------------
# This Plugin Script returns metrics containing information
# about the system's CPU, memory, and kernel.
#
# Example: perl vmStats.pl -sleepTime i -n i
#		sleepTime	seconds to sleep between each command
#				default is 15
#		n		number of times to iterate
#				0 infers no end, value should be positve
#
########################################################################

#import our modules
use FindBin;
use lib ("$FindBin::Bin", "$FindBin::Bin/../lib/perl");
use Wily::PrintMetric;
use Wily::Config;

use strict;
use Getopt::Long;

my $config;

# Get and Check command line arguments here...
#
GetOptions(
	'sleepTime=i'		=> \($config->{'SLEEPTIME'}),
	'n=i'			=> \($config->{'NUMLOOPS'}),
);

my $sleepTime = $config->{'SLEEPTIME'};
my $numLoops = $config->{'NUMLOOPS'};

if ( !defined($numLoops) || $numLoops < 0 ) {
	$numLoops = 0;
}
if ( !defined($sleepTime) || $sleepTime <= 0 ) {
	$sleepTime = 15;
}

my $vmstatCommand = 'vmstat';

my $loops = 0;
while ( $numLoops == 0 || $loops < $numLoops ) {
	# Get the stats
	my @vmstatResults = `$vmstatCommand`;
	# Output on HPUX:
	#         procs           memory                   page    faults       cpu
	#    r     b     w      avm    free   re   at    pi   po    fr   de    sr     in    sy    cs  us sy id
	#    2     0     0     5093   99008    0    0     0    0     0    0     0    120   390    91   2  0 98
	# Get rid of the header lines for each command
	my $vmstatLine = @vmstatResults[2];
	# if there's a trailing \n char, get rid of it
	chomp $vmstatLine;
	my @vmStats = split(" ", $vmstatLine);
	# report the vm stats
	# procs       Information about numbers of processes in various states.
	# r      In run queue
	Wily::PrintMetric::printMetric( type        => 'IntCounter',
                                  resource    => 'VM',
                                  subresource => 'Processes',
                                  name        => 'Running',
                                  value       => int ( $vmStats[0] ),
        );
	# b      Blocked for resources (I/O, paging, etc.)
	Wily::PrintMetric::printMetric( type        => 'IntCounter',
                                  resource    => 'VM',
                                  subresource => 'Processes',
                                  name        => 'Blocked',
                                  value       => int ( $vmStats[1] ),
        );
	# w      Runnable or short sleeper (< 20 secs) but swapped
	Wily::PrintMetric::printMetric( type        => 'IntCounter',
                                  resource    => 'VM',
                                  subresource => 'Processes',
                                  name        => 'Swapped',
                                  value       => int ( $vmStats[2] ),
        );
	# memory      Information about the usage of virtual and real memory.  Virtual pages are considered 
	#             active if they belong to processes that are running or have run in the last 20 seconds.
	# avm     Active virtual pages
	Wily::PrintMetric::printMetric( type        => 'IntCounter',
                                  resource    => 'VM',
                                  subresource => 'Memory',
                                  name        => 'Active virtual (pages)',
                                  value       => int ( $vmStats[3] ),
        );
	# free    Size of the free list
	Wily::PrintMetric::printMetric( type        => 'IntCounter',
                                  resource    => 'VM',
                                  subresource => 'Memory',
                                  name        => 'Free list (pages)',
                                  value       => int ( $vmStats[4] ),
        );
	# page        Information about page faults and paging activity. These are averaged each five seconds,
	#             and given in units per second.
	# re      Page reclaims (without -S)
	Wily::PrintMetric::printMetric( type        => 'IntCounter',
                                  resource    => 'VM',
                                  subresource => 'Paging',
                                  name        => 'Reclaims per sec',
                                  value       => int ( $vmStats[5] ),
        );
	# at      Address translation faults (without -S)
	Wily::PrintMetric::printMetric( type        => 'IntCounter',
                                  resource    => 'VM',
                                  subresource => 'Paging',
                                  name        => 'Address translation faults per sec',
                                  value       => int ( $vmStats[6] ),
        );
	# pi      Pages paged in
	Wily::PrintMetric::printMetric( type        => 'IntCounter',
                                  resource    => 'VM',
                                  subresource => 'Paging',
                                  name        => 'Input per sec',
                                  value       => int ( $vmStats[7] ),
        );
	# po      Pages paged out
	Wily::PrintMetric::printMetric( type        => 'IntCounter',
                                  resource    => 'VM',
                                  subresource => 'Paging',
                                  name        => 'Output per sec',
                                  value       => int ( $vmStats[8] ),
        );
	# fr      Pages freed per second
	Wily::PrintMetric::printMetric( type        => 'IntCounter',
                                  resource    => 'VM',
                                  subresource => 'Paging',
                                  name        => 'Freed per sec',
                                  value       => int ( $vmStats[9] ),
        );
	# de      Anticipated short term memory shortfall
	Wily::PrintMetric::printMetric( type        => 'IntCounter',
                                  resource    => 'VM',
                                  subresource => 'Paging',
                                  name        => 'Anticipated memory shortfall per sec',
                                  value       => int ( $vmStats[10] ),
        );
	# sr      Pages scanned by clock algorithm, per second
	Wily::PrintMetric::printMetric( type        => 'IntCounter',
                                  resource    => 'VM',
                                  subresource => 'Paging',
                                  name        => 'Scanned per sec',
                                  value       => int ( $vmStats[11] ),
        );
	# faults      Trap/interrupt rate averages per second over last 5 seconds.
	# in      Device interrupts per second (nonclock)
	Wily::PrintMetric::printMetric( type        => 'IntCounter',
                                  resource    => 'VM',
                                  subresource => 'Faults',
                                  name        => 'Device interrupts per sec',
                                  value       => int ( $vmStats[12] ),
        );
	# sy      System calls per second
	Wily::PrintMetric::printMetric( type        => 'IntCounter',
                                  resource    => 'VM',
                                  subresource => 'Faults',
                                  name        => 'System calls per sec',
                                  value       => int ( $vmStats[13] ),
        );
	# cs      CPU context switch rate (switches/sec)
	Wily::PrintMetric::printMetric( type        => 'IntCounter',
                                  resource    => 'VM',
                                  subresource => 'Faults',
                                  name        => 'CPU context switches per sec',
                                  value       => int ( $vmStats[14] ),
        );
	# cpu         Breakdown of percentage usage of CPU time for the active processors
	# us      User time for normal and low priority processes
	Wily::PrintMetric::printMetric( type        => 'IntCounter',
                                  resource    => 'VM',
                                  subresource => 'CPU',
                                  name        => 'User (%)',
                                  value       => int ( $vmStats[15] ),
        );
	# sy      System time
	Wily::PrintMetric::printMetric( type        => 'IntCounter',
                                  resource    => 'VM',
                                  subresource => 'CPU',
                                  name        => 'System (%)',
                                  value       => int ( $vmStats[16] ),
        );
	# id      CPU idle
	Wily::PrintMetric::printMetric( type        => 'IntCounter',
                                  resource    => 'VM',
                                  subresource => 'CPU',
                                  name        => 'Idle (%)',
                                  value       => int ( $vmStats[17] ),
        );

	sleep $sleepTime;
	$loops++;
}
